-- Bee. Descended from KrupnoPavel version.

-- ===================================================================

local lcname     = "bee"
local ucname     = "Bee"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =      2       ,
    spawn_chance =  30000       ,

    day_mode     = true         ,
    min_light    =      3       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass"  ,
        "default:dirt_with_grass"      ,
        "earthgen:dirt_with_dry_grass" ,
        "ethereal:green_dirt"          ,
        "ethereal:green_dirt_top"      ,
        "group:flower"                 ,
        "lottmapgen:default_grass"     ,
        "lottmapgen:gondor_grass"      ,
        "lottmapgen:ithilien_grass"    ,
        "lottmapgen:lorien_grass"      ,
        "lottmapgen:mirkwood_grass"    ,
        "lottmapgen:rohan_grass"       ,
        "mg:dirt_with_dry_grass"       ,
        "noisegrid:grass"              ,
    } ,

    add_egg   = false                   ,
    egg_image = msname .. "_inv.png"    ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    passive = true,
    hp_min = 1,
    hp_max = 2,
    armor = 200,
    collisionbox = {-0.2, -0.01, -0.2, 0.2, 0.2, 0.2},
    visual = "mesh",
    mesh = msname .. ".x",
    textures = {
        { msname .. ".png"} ,
    },
    makes_footstep_sound = false,
    sounds = {
        random = msname ,
    },
    walk_velocity = 1,
    jump = true,

    drops = {
        { name="codermobs:honey",
          chance=1, min=1, max=2 } ,
    } ,

    water_damage = 1,
    lava_damage = 1,
    light_damage = 0,
    fall_damage = 0,
    fall_speed = -3,
    animation = {
        speed_normal = 15,
        stand_start = 0,
        stand_end = 30,
        walk_start = 35,
        walk_end = 65,
    },
    on_rightclick = function(self, clicker)
        mobs:capture_mob(self, clicker, 25, 80, 0, true, nil)
    end,
}

-- ===================================================================

-- honey
minetest.register_craftitem ("codermobs:honey", {
    description     = "Honey"                   ,
    inventory_image = "codermobs_honey_inv.png" ,
    on_use          = minetest.item_eat (6)     ,
})
minetest.register_alias ("mobs:honey", "codermobs:honey")

-- ===================================================================
-- Beehive (when placed, spawns bee).

minetest.register_node ("codermobs:beehive", {
    description = "Beehive" ,
    drawtype = "plantlike",
    visual_scale = 1.0 ,
    tiles = {"codermobs_beehive.png"},
    inventory_image = "codermobs_beehive.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = true,

--  groups = {fleshy = 3, dig_immediate = 3},
    groups = {oddly_breakable_by_hand = 3, flammable = 1},
    sounds = default.node_sound_defaults(),

    on_construct = function (pos)
        local meta = minetest.get_meta (pos)
        meta:set_string ("formspec", "size[8,6]"
            .. default.gui_bg
            .. default.gui_bg_img
            .. default.gui_slots
            .. "image[3,0.8;0.8,0.8;codermobs_bee_inv.png]"
            .. "list[current_name;beehive;4,0.5;1,1;]"
            .. "list[current_player;main;0,2.35;8,4;]"
            .. "listring[]")

        meta:get_inventory():set_size("beehive", 1)
    end,

    after_place_node = function (pos, placer, itemstack)
        if placer:is_player() then
            minetest.set_node (pos, {name = "codermobs:beehive", param2 = 1})
            minetest.add_entity (pos, obj_name)
        end
    end,

    on_punch = function (pos, node, puncher)
        -- yep, bee's don't like having their home punched by players
        puncher:set_hp(puncher:get_hp() - 4)
    end,

    allow_metadata_inventory_put = function (pos, listname,
        index, stack, player)
        if listname == "beehive" then
            return 0
        end
        return stack:get_count()
    end ,

    can_dig = function (pos,player)
        local meta = minetest.get_meta(pos)
        -- only dig beehive if no honey inside
        return meta:get_inventory():is_empty ("beehive")
    end,
})
minetest.register_alias ("mobs:beehive", "codermobs:beehive")
minetest.register_alias ("kpgmobs:uley", "codermobs:beehive")


minetest.register_craft ({
    output = "codermobs:beehive",
    recipe = {
        { obj_name, obj_name, obj_name} ,
    }
})

local tame_name = obj_name .. "_set"

minetest.register_craft ({
    output = "codermobs:beehive",
    recipe = {
        { tame_name, tame_name, tame_name } ,
    }
})

-- honey block
minetest.register_node ("codermobs:honey_block", {
    description = "Honey Block",
    tiles = {"codermobs_honey_block.png"},
    groups = {snappy = 3, flammable = 2},
    sounds = default.node_sound_dirt_defaults(),
})
minetest.register_alias ("mobs:honey_block", "codermobs:honey_block")

minetest.register_craft ({
    output = "codermobs:honey_block",
    recipe = {
        { "codermobs:honey", "codermobs:honey", "codermobs:honey" } ,
        { "codermobs:honey", "codermobs:honey", "codermobs:honey" } ,
        { "codermobs:honey", "codermobs:honey", "codermobs:honey" } ,
    }
})

minetest.register_craft ({
    output = "codermobs:honey 9" ,
    recipe = {
        { "codermobs:honey_block" } ,
    }
})

-- ===================================================================

-- beehive workings
minetest.register_abm ({
    nodenames = {"codermobs:beehive"},
    interval  = 6,
    chance    = 6,
    catch_up  = false,
    action   = function (pos, node)
        -- bees only make honey during the day
        local tod = (minetest.get_timeofday() or 0) * 24000
        if tod < 5500 or tod > 18500 then
            return
        end

        -- is hive full?
        local meta  = minetest.get_meta (pos)
        if not meta then return end -- for older beehives
        local inv   = meta:get_inventory()
        local honey = inv:get_stack ("beehive", 1):get_count()

        -- is hive full?
        if honey > 19 then return end

        -- no flowers no honey, nuff said!
        if #minetest.find_nodes_in_area_under_air (
            {x = pos.x - 4, y = pos.y - 3, z = pos.z - 4},
            {x = pos.x + 4, y = pos.y + 3, z = pos.z + 4},
            "group:flower") > 3 then
            inv:add_item ("beehive", "codermobs:honey")
        end
    end
})

-- ===================================================================

codermobs.setup_mob()

mobs:alias_mob ("mobs:bee"        , "codermobs:bee")
mobs:alias_mob ("mobs_animal:bee" , "codermobs:bee")
mobs:alias_mob ("mobs_redo:bee"   , "codermobs:bee")

codermobs.log_done()

-- ===================================================================
-- End of file.
