-- Camel. Descended from "22i-amcaw" mobset.

-- ===================================================================
-- Media license. Applies to model and associated textures.

-- GNU GENERAL PUBLIC LICENSE
-- Version 3, 29 June 2007

-- ===================================================================

local lcname     = "camel"
local ucname     = "Camel"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  20000       ,

    day_mode     =   true       ,
    min_light    =     14       ,
    max_light    =     20       ,
    min_height   =      0       ,
    max_height   =    200       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_grass"       ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "ethereal:grove_dirt"           ,
        "mg:dirt_with_dry_grass"        ,
        "noisegrid:grass"               ,
    } ,

    spawn_by = {
        "default:pine_tree"             ,
        "flowers:mushroom_brown"        ,
    } ,

    add_egg   = true                    ,
    egg_image = "wool_pink.png"         ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {

-- ===================================================================
-- Basic parameters.

    type = mobs_param.spawn_type ,
    makes_footstep_sound = true  ,

    -- Behavior/sight/battle
    armor         = 150          ,
    attack_npcs   = false        ,
    attack_type   = "dogfight"   ,
    damage        =   3          ,
    group_attack  = true         ,
    hp_max        =  35          ,
    hp_min        =  12          ,
    owner_loyal   = false        ,
    passive       = false        ,
    pushable      = true         ,
    reach         =   2.5        ,
    runaway       = true         ,
    view_range    =  10          ,

    -- Velocities
    run_velocity  =   1.5        ,
    walk_velocity =   1.0        ,

    -- Step/jump/fall parameters
    fall_damage   =   1          ,
    fear_height   =   5          ,
    jump          = true         ,
    jump_height   =   2          ,
    stepheight    =   1.2        ,

    -- Environment damage
    lava_damage   =   2          ,
    light_damage  =   0          ,
    water_damage  =   0          ,

    -- Misc. parameters
    floats        =   0          ,

-- ===================================================================
-- Model.

--                    x1     y1     z1     x2    y2    z2
    collisionbox  = { -0.32,  0.00, -0.32, 0.32, 0.50, 0.32 } ,
    mesh          = msname .. ".b3d"                          ,
    rotate        = 180                                       ,
    scale         = 5                                         ,
    visual        = "mesh"                                    ,
    textures      = {{ msname .. "_mesh.png" }}               ,

    animation     = {
        stand_start  = 40 ,
        stand_end    = 80 ,
        walk_start   =  0 ,
        walk_end     = 40 ,
        run_start    =  0 ,
        run_end      = 40 ,
        speed_normal = 25 ,
        speed_run    = 30 ,
    } ,

-- ===================================================================
-- Other.
 
    sounds        = {
    },

    follow        = { "default:apple", "farming:potato" }     ,

    drops         = {
        {
            name   = "animal_materials:meat_venison" ,
            chance = 1, min = 1, max = 1             ,
        } ,
        {
            name   = "animal_materials:antlers"      ,
            chance = 1, min = 1, max = 1             ,
        } ,
        {
            name   = "animal_materials:fur_deer"     ,
            chance = 4, min = 1, max = 1             ,
        } ,
        {
            name   = "animal_materials:bone"         ,
            chance = 4, min = 1, max = 1             ,
        } ,
    } ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame(self, clicker, 8, true, true) then return end
        if mobs:protect(self, clicker) then return end

        if mobs:capture_mob(self, clicker, 0, 5, 50, false, nil) then
            return
        end
    end,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
