-- ===================================================================
-- Description of file.

-- Final MT Cow.
-- Descended from Krupnovpavel-JurajVajda Cow.
-- Final MT version by OldCoder.
-- Edits by Slopsbucket and Greg Yurkon.

-- ===================================================================

local lcname     = "cow"
local ucname     = "Cow"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  40000       ,

    day_mode     = true         ,
    min_light    =     12       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    scale        = 1.25         ,
    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_grass"       ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "ethereal:grove_dirt"           ,
        "mg:dirt_with_dry_grass"        ,
        "noisegrid:grass"               ,
    } ,

    spawn_by     = {
        "group:grass"                   ,
    } ,

    add_egg   = true                    ,
    egg_image = "default_grass.png"     ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type    ,
    makes_footstep_sound = true     ,

    armor         = 200             ,
    attack_npcs   = false           ,
    attack_type   = "dogfight"      ,
    damage        =   4             ,
    fear_height   =   2             ,
    hp_max        =  20             ,
    hp_min        =   5             ,
    stepheight    = 1.2             ,
    jump_height   =   0             ,
    jump          = false           ,
    passive       = false           ,
    pushable      = true            ,
    reach         =   2             ,
    run_velocity  =   2             ,
    view_range    =   8             ,
    walk_velocity =   1             ,
    lava_damage   =   5             ,
    light_damage  =   0             ,
    water_damage  =   1             ,
    floats        =   0             ,
    runaway       = true            ,

    collisionbox  = { -0.40, -0.01, -0.40, 0.40, 1.20, 0.40 } ,
    mesh          = "codermobs_cow.x"                         ,
    visual        = "mesh"                                    ,

    textures = {
        { "codermobs_cow_01.png" } ,
        { "codermobs_cow_02.png" } ,
    },

    sounds = {
        random = "codermobs_cow" ,
    },

    drops = {
        { name="mobs:meat_raw" , chance=1, min=1, max=3 } ,
        { name="mobs:leather"  , chance=1, min=0, max=2 } ,
    } ,

    animation = {
        speed_normal = 15,
        speed_run = 15,
        stand_start = 0,
        stand_end = 30,
        walk_start = 35,
        walk_end = 65,
        run_start = 105,
        run_end = 135,
        punch_start = 70,
        punch_end = 100,
    },

    follow = {"farming:wheat", "default:grass_1"},

    replace_rate = 10,
    replace_what = {
        {"group:grass", "air", 0}

-- Slopsbucket: Suppressed this in June 2019:
--      { "default:dirt_with_grass", "noisegrid:grass", "default:dirt", -1 }
    } ,

    on_rightclick = function(self, clicker)

        -- feed or tame
        if mobs:feed_tame(self, clicker, 8, true, true) then

            -- if fed 7x wheat or grass then cow can be milked again
            if self.food and self.food > 6 then
                self.gotten = false
            end

            return
        end

        if mobs:protect(self, clicker) then return end
        if mobs:capture_mob(self, clicker, 0, 5, 60, false, nil) then return end

        local tool = clicker:get_wielded_item()
        local name = clicker:get_player_name()

-- Jan. 2020: Greg Yurkon:  Fixed  milking  cow with  empty bucket.  I
-- don't know whether the original  should  have worked, since I'm not
-- sure of how aliases work.
--
-- The old version did this:
-- if tool:get_name() == "bucket:bucket_empty" then

        if tool:get_name() == "bucket:empty" then

            --if self.gotten == true
            if self.child == true then
                return
            end

            if self.gotten == true then
                minetest.chat_send_player(name,
                    "Cow already milked!")
                return
            end

            local inv = clicker:get_inventory()

            tool:take_item()
            clicker:set_wielded_item(tool)

            if inv:room_for_item("main", {name = "mobs:bucket_milk"}) then
                clicker:get_inventory():add_item("main", "mobs:bucket_milk")
            else
                local pos = self.object:get_pos()
                pos.y = pos.y + 0.5
                minetest.add_item(pos, {name = "mobs:bucket_milk"})
            end

            self.gotten = true -- milked
            return
        end
    end,

    on_replace = function(self, pos, oldnode, newnode)

        self.food = (self.food or 0) + 1

        -- if cow replaces 8x grass then it can be milked again
        if self.food >= 8 then
            self.food = 0
            self.gotten = false
        end
    end,
}

-- ===================================================================

-- bucket of milk
minetest.register_craftitem(":mobs:bucket_milk", {
    description = "Bucket of Milk",
    inventory_image = "codermobs_bucket_milk.png",
    stack_max = 1,
    on_use = minetest.item_eat(8, 'bucket:bucket_empty'),
    groups = {food_milk = 1, flammable = 3},
})

-- glass of milk
minetest.register_craftitem(":mobs:glass_milk", {
    description = "Glass of Milk",
    inventory_image = "codermobs_glass_milk.png",
    on_use = minetest.item_eat(2, 'vessels:drinking_glass'),
    groups = {food_milk_glass = 1, flammable = 3, vessel = 1 },
})

minetest.register_craft({
    type = "shapeless",
    output = "mobs:glass_milk 4",
    recipe = {
        'vessels:drinking_glass', 'vessels:drinking_glass',
        'vessels:drinking_glass', 'vessels:drinking_glass',
        'mobs:bucket_milk'
    },
    replacements = { {"mobs:bucket_milk", "bucket:bucket_empty"} }
})

minetest.register_craft({
    type = "shapeless",
    output = "mobs:bucket_milk",
    recipe = {
        'mobs:glass_milk', 'mobs:glass_milk',
        'mobs:glass_milk', 'mobs:glass_milk',
        'bucket:bucket_empty'
    },
    replacements = { {"mobs:glass_milk", "vessels:drinking_glass 4"} }
})

-- butter
minetest.register_craftitem(":mobs:butter", {
    description = "Butter",
    inventory_image = "codermobs_butter.png",
    on_use = minetest.item_eat(1),
    groups = {food_butter = 1, flammable = 2},
})

if minetest.get_modpath("farming") and farming and farming.mod then
minetest.register_craft({
    type = "shapeless",
    output = "mobs:butter",
    recipe = {"mobs:bucket_milk", "farming:salt"},
    replacements = {{ "mobs:bucket_milk", "bucket:bucket_empty"}}
})
else -- some saplings are high in sodium so makes a good replacement item
minetest.register_craft({
    type = "shapeless",
    output = "mobs:butter",
    recipe = {"mobs:bucket_milk", "default:sapling"},
    replacements = {{ "mobs:bucket_milk", "bucket:bucket_empty"}}
})
end

-- cheese wedge
minetest.register_craftitem(":mobs:cheese", {
    description = "Cheese",
    inventory_image = "codermobs_cheese.png",
    on_use = minetest.item_eat(4),
    groups = {food_cheese = 1, flammable = 2},
})

minetest.register_craft({
    type = "cooking",
    output = "mobs:cheese",
    recipe = "mobs:bucket_milk",
    cooktime = 5,
    replacements = {{ "mobs:bucket_milk", "bucket:bucket_empty"}}
})

-- cheese block
minetest.register_node(":mobs:cheeseblock", {
    description = "Cheese Block",
    tiles = {"codermobs_cheeseblock.png"},
    is_ground_content = false,
    groups = {crumbly = 3},
    sounds = default.node_sound_dirt_defaults()
})

minetest.register_craft({
    output = "mobs:cheeseblock",
    recipe = {
        {'mobs:cheese', 'mobs:cheese', 'mobs:cheese'},
        {'mobs:cheese', 'mobs:cheese', 'mobs:cheese'},
        {'mobs:cheese', 'mobs:cheese', 'mobs:cheese'},
    }
})

minetest.register_craft({
    output = "mobs:cheese 9",
    recipe = {
        {'mobs:cheeseblock'},
    }
})

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("mobs:cow"        , obj_name)
mobs:alias_mob ("mobs_animal:cow" , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
