-- ===================================================================

local dgspawn_log = minetest.setting_getbool ("dgspawn_log")

-- ===================================================================

local                          bat_maxnum         =
    codermobs.gnsnil ("dgspawn_bat_maxnum"        )
if                             bat_maxnum         == nil then
                               bat_maxnum         = 2
end
if bat_maxnum        > 10 then bat_maxnum        = 10 end


local                          mdskeleton_maxnum  =
    codermobs.gnsnil ("dgspawn_mdskeleton_maxnum" )
if                             mdskeleton_maxnum  == nil then
                               mdskeleton_maxnum  = 1
end
if mdskeleton_maxnum > 10 then mdskeleon_maxnum = 10 end

local                          oerkki_maxnum      =
    codermobs.gnsnil ("dgspawn_oerkki_maxnum"     )
if                             oerkki_maxnum      == nil then
                               oerkki_maxnum      = 0
end
if oerkki_maxnum     > 10 then oerkki_maxnum      = 10 end

local                          seconds            =
    codermobs.gnsnil ("dgspawn_seconds"           )
if                             seconds            == nil then
                               seconds            = 30
end
if (seconds > 0) and (seconds < 10) then seconds  = 10 end

if dgspawn_log then
    minetest.log ("action",
"dgspawn: " ..
    "bat_maxnum="        .. bat_maxnum        .. " " ..
    "mdskeleton_maxnum=" .. oerkki_maxnum     .. " " ..
    "oerkki_maxnum="     .. mdskeleton_maxnum .. " " ..
    "seconds="           .. seconds)
end

-- ===================================================================

local spawn_node = "codermobs:dungeon_spawner"

minetest.register_node (spawn_node, {
    description       = "Dungeon Mob Spawner"   ,
    tiles             = { "default_stone.png" } ,
    drop              = ""                      ,
    is_ground_content = true                    ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================

local function place_spawner (tab)

    if tab == nil or #tab < 1 then return end
    local rlist = {}

    for room = 1,#tab do
        table.insert (rlist, math.random (1, #rlist+1), room)
    end
    
    for room = 1,#tab do
        local pos = tab [rlist [room]]

        local npos1 = { x=pos.x, y=pos.y-1, z=pos.z }
        local npos2 = { x=pos.x, y=pos.y-0, z=pos.z }

        local node1 = core.get_node_or_nil (npos1)
        local node2 = core.get_node_or_nil (npos2)

        if  node1 and node1.name ~= "air" and
            node2 and node2.name == "air" then

            if dgspawn_log then
                minetest.log ("action", "dgspawn 001: " ..
pos.x .. "," .. pos.y .. "," .. pos.z)
            end

            core.set_node (npos2, {name = spawn_node})
            return
        end
    end

    for room = 1,#tab do
        local pos = tab [room]

        for ii = -1,1 do
            for jj = -1,1 do
                local npos1 = { x=pos.x + ii, y=pos.y-1, z=pos.z+jj }
                local npos2 = { x=pos.x + ii, y=pos.y-0, z=pos.z+jj }

                local node1 = core.get_node_or_nil (npos1)
                local node2 = core.get_node_or_nil (npos2)

                if node1 and node1.name ~= "air" and
                   node2 and node2.name == "air" then

                    if dgspawn_log then
                        minetest.log ("action", "dgspawn 002: " ..
pos.x .. "," .. pos.y .. "," .. pos.z)
                    end

                    core.set_node (npos2, {name = spawn_node})
                    return
                end
            end
        end
    end
end

core.set_gen_notify ("dungeon")

core.register_on_generated (function (minp, maxp, blockseed)
    local ntf = core.get_mapgen_object ("gennotify")
    if ntf and ntf.dungeon and #ntf.dungeon >= 1 then
        core.after (3, place_spawner, table.copy (ntf.dungeon))
    end
end)

if seconds > 0 then
minetest.register_abm ({
    nodenames = { spawn_node } ,
    interval  = seconds        ,
    chance    =  1             ,
    catch_up  = false          ,

    action    = function (pos, node)
        local mobname
        if dgspawn_log then
            minetest.log ("action",
"dgspawn: ABM at " .. pos.x .. "," .. pos.y .. "," .. pos.z)
        end

        mobname = "codermobs:bat"
        if  codermobs.mob_enabled (mobname) and
            mobs.count_mobs (pos, mobname, 1)
                <= bat_maxnum     then
            minetest.add_entity (pos, mobname)
        end

        mobname = "codermobs:oerkki"
        if  codermobs.mob_enabled (mobname) and
            mobs.count_mobs (pos, mobname, 1)
                <= oerkki_maxnum  then
            minetest.add_entity (pos, mobname)
        end

        mobname = "codermobs:mdskeleton"
        if  codermobs.mob_enabled (mobname) and
            mobs.count_mobs (pos, mobname, 1) <=
                mdskeleton_maxnum then
            minetest.add_entity (pos, mobname)
        end
    end
})
end

-- ===================================================================
-- End of file.
