-- Kitten. Descended from Jordach-BFD version.
-- Changed by Greg Yurkon.

-- ===================================================================

local lcname     = "kitten"
local ucname     = "Kitten"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  80000       ,

    day_mode     = true         ,
    min_light    =     10       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass"   ,
        "default:dirt_with_grass"       ,
        "earthgen:dirt_with_dry_grass"  ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "ethereal:grove_dirt"           ,
        "mg:dirt_with_dry_grass"        ,
        "noisegrid:grass"               ,
    } ,

    add_egg   = false                   ,
    egg_image = msname .. "_inv.png"    ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = false ,

    -- Most behavior/sight/battle
    armor          = 200         ,
    attack_odds    =  85         ,
    -- Added attack_type. Greg Yurkon.
    attack_type    = "dogfight"  ,
    reach          =   1         ,
    damage         =   1         ,
    hp_max         =  10         ,
    hp_min         =   5         ,
    passive        = false       ,
    runaway        = true        ,
    view_range     =  10         ,

    -- Velocities
    run_velocity   =   2         ,
    walk_velocity  =   0.6       ,

    -- Step/jump/fall parameters
    jump_height    =   0         ,
    jump           = false       ,
    stepheight     =   1.2       ,

    -- Environment damage
    lava_damage    =   5         ,
    water_damage   =   1         ,

    -- Misc. parameters
    floats         =   0         ,

    collisionbox     = { -0.30, -0.30, -0.30, 0.30, 0.10, 0.30 } ,
    mesh             = msname .. ".b3d"                          ,
    visual           = "mesh"                                    ,
    visual_size      = { x = 0.5, y = 0.5 }                      ,

    textures         = {
        { "codermobs_kitten_striped.png"  } ,
        { "codermobs_kitten_splotchy.png" } ,
        { "codermobs_kitten_ginger.png"   } ,
        { "codermobs_kitten_sandy.png"    } ,
    } ,

    animation        = {
        walk_start   =   0 ,
        walk_end     =  96 ,
        stand_start  =  97 ,
        stand_end    = 192 ,
        speed_normal =  42 ,
    } ,

    drops            = {
        { name="farming:string", chance=1, min=1, max=1 } ,
    } ,

    follow           = {
        "codermobs:rat"        ,
        "codermobs:rat_better" ,
        "ethereal:fish_raw"    ,
        "kpgmobs:whiterat"     ,
        "mobs:rat"             ,
        "mobs_better_rat:rat"  ,
        "oldmobs:rat"          ,
    } ,

    specific_attack  = codermobs.make_mob_list ({
        "rat" , "rat_better" ,
    }) ,

    sounds           = { random = msname } ,

--I enclosed the feed_tame call in an if statement and added a second 
--true argument. I'm not sure whether those were necessary. I just
--saw that other mobs were done that way. Greg Yurkon.
    on_rightclick    = function (self, clicker)
        if mobs:feed_tame(self, clicker, 4, true, true) then
            return
        end
        mobs:capture_mob (self, clicker, 50, 50, 90, false, nil)
    end ,
}

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("codermobs:cat"      , obj_name)
mobs:alias_mob ("mobs:kitten"        , obj_name)
mobs:alias_mob ("mobs_animal:kitten" , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
