-- Silverfish. Descended from "mobs_mc" version.

-- ===================================================================

local lcname     = "silverfish"
local ucname     = "Silverfish"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

-- Most of these parameters  are  presently unused for this mob.  How-
-- ever, these parameters are used:
--
-- max_height, min_height, spawn_type, add_egg, egg_image

    aoc          = -1           ,
    day_mode     = true         ,
    min_light    =  0           ,
    max_light    = 10           ,
    min_height   = -31000       ,
    max_height   =  31000       ,
    spawn_chance =  10000       ,
    spawn_type   = "monster"    ,

    -- This mob uses a special type of spawning
    spawn_nodes  = {
    } ,

    add_egg   = true                        ,
    egg_image = msname .. "_spawn.png"      ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = false ,

    -- Most behavior/sight/battle
    attack_type    = "dogfight"  ,
    damage         =    1        ,
    group_attack   = true        ,
    hp_max         =    8        ,
    hp_min         =    8        ,
    passive        = false       ,
    pathfinding    = true        ,
    reach          =    1        ,
    runaway        = true        ,
    view_range     =   16        ,

    -- Velocities
    run_velocity   =    2        ,
    walk_velocity  =    0.6      ,

    -- Step/jump/fall parameters
    fear_height    =    4        ,
    jump           = false       ,
    jump_height    =    0        ,
    stepheight     =    1.2      ,

    -- Environment damage
    lava_damage    =    4        ,
    light_damage   =    0        ,
    water_damage   =    1        ,

    -- Misc. parameters
    blood_amount   =    0        ,
    floats         =    0        ,

    collisionbox     = { -0.40, -0.01, -0.40, 0.40, 0.44, 0.40 } ,
    mesh             = msname .. ".b3d"                          ,
    visual           = "mesh"                                    ,
    visual_size      = { x=3, y=3 }                              ,
    textures         = {{ msname .. ".png" }}                    ,

    animation        = {
        stand_start  =  0 ,
        stand_end    = 20 ,
        walk_start   =  0 ,
        walk_end     = 20 ,
        run_start    =  0 ,
        run_end      = 20 ,
        speed_normal = 25 ,
        speed_run    = 50 ,
    } ,
}

-- ===================================================================

silverfish               = {}
silverfish.is_enabled    = codermobs_enable_thismob
codermobs_enable_thismob = false

codermobs.setup_mob()

-- ===================================================================

local spawn_silverfish = function (pos, oldnode, oldmetadata, digger)
    if silverfish.is_enabled and
       not minetest.setting_getbool ("creative_mode") then
        minetest.add_entity (pos, obj_name)
    end
end

-- ===================================================================

minetest.register_node ("codermobs:monster_egg_stone", {
    description       = "Stone Monster Egg"     ,
    tiles             = { "default_stone.png" } ,
    after_dig_node    = spawn_silverfish        ,
    drop              = ""                      ,
    is_ground_content = true                    ,

    groups = {
        oddly_breakable_by_hand = 2 ,
        spawns_silverfish       = 1 ,
    } ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================

minetest.register_node ("codermobs:monster_egg_cobble", {
    after_dig_node    = spawn_silverfish          ,
    description       = "Cobblestone Monster Egg" ,
    drop              = ""                        ,
    tiles             = { "default_cobble.png" }  ,
    is_ground_content = false                     ,

    groups = {
        oddly_breakable_by_hand = 2 ,
        spawns_silverfish       = 1 ,
    } ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================

minetest.register_node ("codermobs:monster_egg_mossycobble", {
    after_dig_node    = spawn_silverfish                ,
    description       = "Mossy Cobblestone Monster Egg" ,
    drop              = ""                              ,
    tiles             = { "default_mossycobble.png" }   ,
    is_ground_content = false                           ,

    groups = {
        oddly_breakable_by_hand = 2 ,
        spawns_silverfish       = 1 ,
    } ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================

minetest.register_node ("codermobs:monster_egg_stonebrick", {
    after_dig_node    = spawn_silverfish              ,
    description       = "Stone Brick Monster Egg"     ,
    drop              = ""                            ,
    is_ground_content = false                         ,
    paramtype2        = "facedir"                     ,
    place_param2      = 0                             ,
    tiles             = { "default_stone_brick.png" } ,

    groups = {
        oddly_breakable_by_hand = 2 ,
        spawns_silverfish       = 1 ,
    } ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================

minetest.register_node ("codermobs:monster_egg_stone_block", {
    after_dig_node    = spawn_silverfish              ,
    description       = "Stone Block Monster Egg"     ,
    drop              = ""                            ,
    tiles             = { "default_stone_block.png" } ,
    is_ground_content = false                         ,

    groups = {
        oddly_breakable_by_hand = 2 ,
        spawns_silverfish       = 1 ,
    } ,

    sounds = default.node_sound_stone_defaults() ,
})

-- ===================================================================
-- Randomly generate stone monster eggs.

local mg_name = minetest.get_mapgen_setting ("mg_name")
local y_max   = mobs_param.max_height
local y_min   = mobs_param.min_height

local cl_num_ores = 3
local cl_scarcity = 16 * 16 * 16
local cl_size     = 2

if mg_name == "v6" then
    cl_scarcity = 28 * 28 * 28
end

if enable_wonder then
    cl_num_ores = 1
    cl_scarcity = 12 * 12 * 12
    cl_size     = 1
end

if not silverfish.is_enabled then
    cl_scarcity = 100 * 100 * 100
    y_min       = -10001
    y_max       = -10000
end

minetest.register_ore ({
    ore_type       = "scatter"                     ,
    wherein        = "default:stone"               ,

    ore            = "codermobs:monster_egg_stone" ,
    clust_scarcity = cl_scarcity                   ,
    clust_num_ores = cl_num_ores                   ,
    clust_size     = cl_size                       ,
    y_min          = y_min                         ,
    y_max          = y_max                         ,
})

-- ===================================================================

codermobs.log_done()

-- ===================================================================
-- End of file.
