-- ===================================================================

codermobs.msget = minetest.setting_get

-- ===================================================================

codermobs.numset_if_any = function (name)
    local  n = tonumber (minetest.setting_get (name))
    return n
end

-- ===================================================================

codermobs.getnumsetting = function (name)
    local  n = tonumber (minetest.setting_get (name)) or 0
    return n
end

-- ===================================================================

codermobs.getnumsetting_or_nil = function (name)
    local  n = tonumber (minetest.setting_get (name)) or nil
    return n
end

codermobs.gnsnil = codermobs.getnumsetting_or_nil

-- ===================================================================

codermobs.starts_with = function (String, Start)
    return string.sub (String, 1,
        string.len (Start)) == Start
end

-- ===================================================================

codermobs.starts_with_any_of = function (String, StartList)
    for _, str in ipairs (StartList) do
        if string.sub (String, 1, string.len (str)) == str then
            return true
        end
    end
    return false
end

-- ===================================================================

codermobs.mob_enabled = function (lcname)

    if   codermobs.starts_with (lcname, "codermobs:") then
        lcname   = string.gsub (lcname, "codermobs:", "", 1)
    end

    local eflag =
        (codermobs.default_mobs [lcname] ~= nil) or
        minetest.setting_getbool ("codermobs_enable_"  .. lcname) or
        minetest.setting_getbool ("enable_codermobs_"  .. lcname) or
        codermobs_enable_all

    local dflag =
        minetest.setting_getbool ("codermobs_disable_" .. lcname) or
        minetest.setting_getbool ("disable_codermobs_" .. lcname) or
        codermobs_disable_all

    if dflag then eflag = false end
    return eflag
end

-- ===================================================================

codermobs.make_mob_list = function (oldlist)
    local newlist = {}
    for _, name in ipairs (oldlist) do
        if string.find (name, ":") == nil then
            name = "codermobs:" .. name
        end
        table.insert (newlist, name)
    end
    return newlist
end

-- ===================================================================

codermobs.make_mob_set = function (list)
    local set = {}
    for _, name in ipairs (list) do
        local full_name = "codermobs:" .. name
        set [full_name] = true
    end
    return set
end

-- ===================================================================

codermobs.adjust_param = function()

    local xyz
    local gnset  = codermobs.getnumsetting
    local lcname = mobs_param.lcname
    local stype  = mobs_param.spawn_type

-- ===================================================================

    codermobs_enable_thismob         = codermobs.mob_enabled (lcname)

    local is_regional                =
        minetest.setting_getbool ("codermobs_regional_" .. lcname) or
        minetest.setting_getbool ("regional_codermobs_" .. lcname)

    if is_regional then
        codermobs_enable_thismob     = true
    end

    codermobs_monster_thismob        =
        minetest.setting_getbool ("codermobs_monster_"  .. lcname) or
        minetest.setting_getbool ("monster_codermobs_"  .. lcname)

    codermobs_peaceful_thismob       =
        minetest.setting_getbool ("codermobs_peaceful_" .. lcname) or
        minetest.setting_getbool ("peaceful_codermobs_" .. lcname)

-- ===================================================================

    if (stype == "monster") or (stype == "animal") then
        if  codermobs_monster_global  or
            codermobs_monster_thismob then
            codermobs_monster_thismob = true
            stype                     = "monster"
            mobs_param.spawn_type     = stype
        end
        if codermobs_peaceful_global  or
           codermobs_peaceful_thismob then
            codermobs_monster_thismob = false
            stype                     = "animal"
            mobs_param.spawn_type     = stype 
        end
        if  stype == "monster" and
            codermobs_disable_monsters then
            codermobs_enable_thismob = false
        end
    end

-- ===================================================================

        xyz = gnset ("codermobs_minheight_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_minheight_global")
    end
    if  xyz ~= 0 then
        mobs_param.min_height = xyz
    end
        xyz = gnset ("codermobs_maxheight_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_maxheight_global")
    end
    if  xyz ~= 0 then
        mobs_param.max_height = xyz
    end

-- ===================================================================

        xyz = gnset ("codermobs_minlight_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_minlight_global")
    end
    if  xyz ~= 0 then
        mobs_param.min_light = xyz
    end
        xyz = gnset ("codermobs_maxlight_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_maxlight_global")
    end
    if  xyz ~= 0 then
        mobs_param.max_light = xyz
    end

-- ===================================================================

        xyz = gnset ("codermobs_aoc_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_aoc_global")
    end
    if  xyz ~= 0 then
        mobs_param.aoc = xyz
    end

-- ===================================================================

        xyz = gnset ("codermobs_chance_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_chance_global")
    end
    if  xyz ~= 0 then
        mobs_param.spawn_chance = xyz
    end

-- ===================================================================

        xyz = gnset ("codermobs_mbrange_" .. lcname)
    if  xyz == 0 then
        xyz = gnset ("codermobs_mbrange_global")
    end
    if  xyz ~= 0 then
        mobs_param.mbrange = xyz
    end
end

-- ===================================================================

codermobs.setup_mob = function()

-- RJK June 2019:  Fix a problem where  mobs lose their textures below
-- sea level.

    if mobs_param.core_param.collisionbox [2] == 0 then
       mobs_param.core_param.collisionbox [2] =
       0                                      - 0.01
       mobs_param.core_param.collisionbox [5] =
       mobs_param.core_param.collisionbox [5] - 0.01
    end

    if  mobs_param.scale == nil and
                           mobs_param.core_param.scale ~= nil then
        mobs_param.scale = mobs_param.core_param.scale
    end

    if mobs_param.scale and mobs_param.scale > 0 then
        local ss = mobs_param.scale

        if  mobs_param.core_param.visual_size == nil then
            mobs_param.core_param.visual_size = { x = 1 , y = 1 , }
        end

        mobs_param.core_param.visual_size.x =
        mobs_param.core_param.visual_size.x * ss
        mobs_param.core_param.visual_size.y =
        mobs_param.core_param.visual_size.y * ss
        local newbox = {}

        for ii, nn in ipairs (mobs_param.core_param.collisionbox) do
            newbox [ii] = nn * ss
        end

        mobs_param.scale = nil
        mobs_param.core_param.collisionbox = newbox
    end

    mobs:register_mob (mobs_param.obj_name ,
                       mobs_param.core_param)

-- Flags are used here intentionally to disable  just spawning and not
-- the entire module.  This makes it possible for  MT server admins to
-- turn  mobs on and off  without ending up with lots of unknown enti-
-- ties.

    if not codermobs_disable_all  and
        codermobs_enable_thismob  then

        mobs.amphibious = mobs_param.amphibious
        mobs.mbrange    = mobs_param.mbrange
        mobs.spawn_by   = mobs_param.spawn_by

        mobs:register_spawn (mobs_param.obj_name,
            mobs_param.spawn_nodes  ,
            mobs_param.max_light    ,
            mobs_param.min_light    ,
            mobs_param.spawn_chance ,
            mobs_param.aoc          ,
            mobs_param.max_height   ,
            mobs_param.day_mode     )

        mobs.amphibious = nil
        mobs.mbrange    = nil
        mobs.spawn_by   = nil

        if codermobs_verbose then
            minetest.log ("action", mobs_param.obj_name
                .. " spawn registered"
                .. " chance:"   .. mobs_param.spawn_chance
                .. " aoc:"      .. mobs_param.aoc
            )
        end
    end

    mobs:register_egg (mobs_param.obj_name  ,
                       mobs_param.ucname    ,
                       mobs_param.egg_image ,
                       mobs_param.add_egg   )
end

-- ===================================================================

codermobs.log_done = function()
    if  codermobs.log_mob_loaded and
        mobs_param        ~= nil and
        mobs_param.lcname ~= nil then
        ocutil.log ("codermobs: loaded " .. mobs_param.lcname)
    end

    mobs_param = nil
end

-- ===================================================================

local param_deletemobs = {
    description = "Delete CoderMobs in a specified radius" ,
    params      = "radius (optional)"  ,
    privs       = { worldedit = true } ,

    func = function (plname, params)
        local radius = tonumber (params)
        if radius == nil then radius = 128 end
        if radius <   1  then radius =   1 end
        if radius > 512  then radius = 512 end

        local player = minetest.env:get_player_by_name (plname)
        local pos    = player:getpos()
        local ent    = nil
        local tnob   = minetest.get_objects_inside_radius (pos, radius)
        local nnob   = table.getn (tnob)

        if nnob == 0 then return end
        local numdel = 0

        for foo, obj in ipairs (tnob) do
            local  tamed = false
            if obj.tamed ~= nil then tamed = obj.tamed end

            ent = obj:get_luaentity()
            if ent ~= nil and not tamed then
                if ent.tamed ~= nil then tamed = obj.tamed end
                local name = ent.name
                if name == nil then name = "" end

                if not tamed and codermobs.starts_with_any_of (name,
                {
                    "codermobs:" , "codersea:" , "codershark:" ,
                    "critters:"  , "petores:"  , "whinny:"     ,
                }) then
                    numdel = numdel + 1
                    obj:remove()
                end
            end
        end

        local info = "Deleted " .. numdel .. " mob"
        if numdel ~= 1 then info = info .. "s" end
        minetest.chat_send_player (plname, info)
        minetest.log ("action", info)
    end
}

minetest.register_chatcommand ("deletecodermobs" , param_deletemobs)
minetest.register_chatcommand ("deletemobs"      , param_deletemobs)

-- ===================================================================

local param_listmobs = {
    description = "List CoderMobs in a specified radius" ,
    params      = "radius (optional)" ,
    privs       = {}                  ,

    func = function (plname, params)
        local radius = tonumber (params)
        if radius == nil then radius =  32 end
        if radius <   1  then radius =   1 end
        if radius > 128  then radius = 128 end

        local player = minetest.env:get_player_by_name (plname)
        local pos    = player:getpos()
        local ent    = nil
        local tnob   = minetest.get_objects_inside_radius (pos, radius)
        local nnob   = table.getn (tnob)

        minetest.log ("action", "mobs:")
        if nnob == 0 then return end

        for foo, obj in ipairs (tnob) do
            ent = obj:get_luaentity()
            if ent ~= nil then
                local name = ent.name
                if name == nil then name = "noname" end

                if codermobs.starts_with_any_of (name,
                {
                    "codermobs:" , "critters:"   , "petores:" ,
                    "whinny:"    , "codershark:" ,
                }) then
                    local he = ent.health
                    local lt = ent.lifetimer
                    if he == nil then he = "?"   end
                    if lt == nil then lt = -1000 end
                    lt = math.floor (lt)

                    local info = "mob"
                        .. " "  .. name
                        .. " H" .. he
                        .. " L" .. lt
                    minetest.chat_send_player (plname, info)
                    minetest.log ("action", info)
                end
            end
        end
    end
}

minetest.register_chatcommand ("listcodermobs" , param_listmobs)
minetest.register_chatcommand ("listmobs"      , param_listmobs)

-- ===================================================================

-- From NSSM:
--
codermobs.explosion_particles = function (pos, exp_radius)
    minetest.add_particlespawner (
        100*exp_radius/2, --amount
        0.1, --time
        {x=pos.x-exp_radius, y=pos.y-exp_radius, z=pos.z-exp_radius}, --minpos
        {x=pos.x+exp_radius, y=pos.y+exp_radius, z=pos.z+exp_radius}, --maxpos
        {x=0, y=0, z=0}, --minvel
        {x=0.1, y=0.3, z=0.1}, --maxvel
        {x=-0.5,y=1,z=-0.5}, --minacc
        {x=0.5,y=1,z=0.5}, --maxacc
        0.1, --minexptime
        4, --maxexptime
        6, --minsize
        12, --maxsize
        false, --collisiondetection
        "codermobs_tnt_smoke.png" --texture
    )
end

-- ===================================================================

minetest.register_node ("codermobs:can_oil", {
    description = "Can of Motor Oil"                  ,
    tiles       = { "can_oil.jpg" }                   ,
    sounds      = default.node_sound_stone_defaults() ,

    groups      = {
        cracky=2, oddly_breakable_by_hand=3, flammable=3
    } ,
})

-- ===================================================================
-- End of file.
